@@ -6,7 +6,7 @@
 
 #  Generator function of class fd
 
-fd <- function (coef=NULL, basisobj=NULL, fdnames=defaultnames)
+fd <- function (coef=NULL, basisobj=NULL, fdnames=NULL)
 {
   #  This function creates a functional data object.
   #    A functional data object consists of a basis for expanding a functional
@@ -38,8 +38,8 @@ fd <- function (coef=NULL, basisobj=NULL, fdnames=defaultnames)
   #  Returns:
   #  FD ... a functional data object
 
-  #  Last modified 2008.10.16 by Spencer Graves
-  #  Previously modified 28 September 2008 by Jim Ramsay
+  #  Last modified 2008.12.26 by Spencer Graves
+  #  Previously modified 2008.10.16 by Spencer Graves
 
 ##
 ## 1.  check coef and get its dimensions
@@ -105,39 +105,57 @@ fd <- function (coef=NULL, basisobj=NULL, fdnames=defaultnames)
 # If dropind is not trapped earlier, it will generate the following
 # cryptic error message:
   if (coefd[1] != basisobj$nbasis)
-    stop("Number of coefficients does not match number of basis functions.")
+    stop("Number of coefficients does not match ",
+         "the number of basis functions.")
 
-    #  setup number of replicates and number of variables
-
-    if (ndim > 1) nrep <- coefd[2] else nrep <- 1
-    if (ndim > 2) nvar <- coefd[3] else nvar <- 1
+#  setup number of replicates and number of variables
 
+  if (ndim > 1) nrep <- coefd[2] else nrep <- 1
+  if (ndim > 2) nvar <- coefd[3] else nvar <- 1
+##
+## 3.  fdnames & dimnames(coef)
+##
     #  set up default fdnames
 
-    if (ndim == 1) defaultnames <- list("time", "reps", "values")
-    if (ndim == 2) defaultnames <- list("time",
-                                        paste("reps",as.character(1:nrep)),
-                                        "values")
-    if (ndim == 3) defaultnames <- list("time",
-                                        paste("reps",as.character(1:nrep)),
-                                        paste("values",as.character(1:nvar)) )
+  if(is.null(fdnames)){
+    if (ndim == 1) fdnames <- list("time", "reps", "values")
+    if (ndim == 2) fdnames <- list("time",
+            paste("reps",as.character(1:nrep)), "values")
+    if (ndim == 3) fdnames <- list("time",
+            paste("reps",as.character(1:nrep)),
+            paste("values",as.character(1:nvar)) )
 
-    names(defaultnames) <- c("args", "reps", "funs")
+    names(fdnames) <- c("args", "reps", "funs")
+  }
+
+  if(is.null(dimnames(coef))){
+    dimc <- dim(coef)
+    ndim <- length(dimc)
+    dnms <- vector('list', ndim)
+    if(dimc[1] == length(fdnames[[1]]))
+      dnms[[1]] <- fdnames[[1]]
+    if((ndim>1) && (dimc[2]==length(fdnames[[2]])))
+      dnms[[2]] <- fdnames[[2]]
+    if((ndim>2) && (dimc[3]==length(fdnames[[3]])))
+      dnms[[3]] <- fdnames[[3]]
+    if(!all(sapply(dnms, is.null)))
+      dimnames(coef) <- dnms
+  }
 
 #  S4 definition
 #   fdobj <- new("fd", coefs=coef, basis=basisobj, fdnames=fdnames)
 
 #  S3 definition
 
-    fdobj <- list(coefs=coef, basis=basisobj, fdnames=fdnames)
+  fdobj <- list(coefs=coef, basis=basisobj, fdnames=fdnames)
     oldClass(fdobj) <- "fd"
 
     fdobj
 }
 
-#  -------------------------------------------------------------------------
+#  ------------------------------------------------------------------
 #  "print" method for "fd"
-#  -------------------------------------------------------------------------
+#  ------------------------------------------------------------------
 
 print.fd <- function(x, ... )
 {
@@ -151,9 +169,9 @@ print.fd <- function(x, ... )
 
 }
 
-#  -------------------------------------------------------------------------
+#  ------------------------------------------------------------------
 #  "summary" method for "fd"
-#  -------------------------------------------------------------------------
+#  ------------------------------------------------------------------
 
 summary.fd <- function(object,...)
 {
@@ -173,7 +191,7 @@ summary.fd <- function(object,...)
   plus.fd(e1, e2)
 }
 
-plus.fd <- function(e1, e2, basisobj=basisobj1*basisobj2)
+plus.fd <- function(e1, e2, basisobj=NULL)
 {
 #  PLUS: Pointwise sum of two functional data objects,
 #    the sum of a scalar and a functional data object,
@@ -197,13 +215,13 @@ plus.fd <- function(e1, e2, basisobj=basisobj1*basisobj2)
 #  coefficient matrices are the same dims, the coefficient
 #  matrices are simply added.
 
-#  last modified 2008.09.30 by Giles Hooker
-#  Previously modified 3 January 2007
+# Last modified 2008.12.26 by Spencer Graves
+# Previously modified 2008.09.30 by Giles Hooker
 
-if (!(inherits(e1, "fd") || inherits(e2, "fd")))
+  if (!(inherits(e1, "fd") || inherits(e2, "fd")))
       stop("Neither argument for + is a functional data object.")
 
-if (inherits(e1, "fd") && inherits(e2, "fd")) {
+  if (inherits(e1, "fd") && inherits(e2, "fd")) {
     #  both arguments are functional data objects
     #  check to see of the two bases are identical
     #  and if (the coefficient matrices are conformable.
@@ -224,104 +242,105 @@ if (inherits(e1, "fd") && inherits(e2, "fd")) {
     #  test to see if the two objects match completely
     if (basisobj1 == basisobj2) {
         #  the two coefficient matrices can be simply added
-        fdnames <- e1$fdnames
-        plusfd  <- fd(coef1 + coef2, basisobj1, fdnames)
-        return(plusfd)
+      fdnames <- e1$fdnames
+      plusfd  <- fd(coef1 + coef2, basisobj1, fdnames)
+      return(plusfd)
     }
+    basisobj <- basisobj1 * basisobj2
     #  check to see if (the number of dimensions match
     ndim1  <- length(coefd1)
     ndim2  <- length(coefd2)
     if (ndim1 != ndim2)
-        stop("Dimensions of coefficient matrices not compatible.")
+      stop("Dimensions of coefficient matrices not compatible.")
     #  allow for one function being a single replicate,
     #  and if (so, copy it as many times as there are replicates
     #  in the other function.
     if (coefd1[2] == 1 && coefd2[2] > 1) {
-        if      (ndim1 == 2) coef1 <- outer(coef1,rep(1,coefd2[2]))
-        else if (ndim1 == 3) {
-            temp <- array(0,coefd2)
-            for (j in 1:coefd1[3])
-                temp[,,j] <- outer(coef1[,1,j],rep(1,coefd2[2]))
-            coef1 <- temp
-        } else
-            stop("Dimensions of coefficient matrices not compatible.")
-        coefd1 <- dim(coef1)
-        e1$coefs <- coef1
+      if      (ndim1 == 2) coef1 <- outer(coef1,rep(1,coefd2[2]))
+      else if (ndim1 == 3) {
+        temp <- array(0,coefd2)
+        for (j in 1:coefd1[3])
+          temp[,,j] <- outer(coef1[,1,j],rep(1,coefd2[2]))
+        coef1 <- temp
+      } else
+      stop("Dimensions of coefficient matrices not compatible.")
+      coefd1 <- dim(coef1)
+      e1$coefs <- coef1
     }
     if (coefd1[2] >  1 && coefd2[2] == 1) {
-        if      (ndim2 == 2) coef2 <- outer(coef2,rep(1,coefd1[2]))
-        else if (ndim1 == 3) {
+      if      (ndim2 == 2) coef2 <- outer(coef2,rep(1,coefd1[2]))
+      else if (ndim1 == 3) {
 #            temp <- zeros(coefd1)
-            temp <- array(0, dim=coefd1)
-            for (j in 1:coefd2[3])
+        temp <- array(0, dim=coefd1)
+        for (j in 1:coefd2[3])
 #                temp[,,j] <- squeeze(coef2[,1,j])*ones(1,coefd1[2])
-                temp[,,j] <- (coef2[, 1, j, drop=TRUE] %o% rep(1, coefd1[2]))
-            coef2 <- temp
-        } else
-            stop("Dimensions of coefficient matrices not compatible.")
-        coefd2 <- dim(coef2)
-        e2$coefs <- coef2
+          temp[,,j] <- (coef2[, 1, j, drop=TRUE] %o% rep(1, coefd1[2]))
+        coef2 <- temp
+      } else
+      stop("Dimensions of coefficient matrices not compatible.")
+      coefd2 <- dim(coef2)
+      e2$coefs <- coef2
     }
     #  check for equality of dimensions of coefficient matrices
     if (coefd1[2] != coefd2[2])
-        stop("Number of replications are not equal.")
+      stop("Number of replications are not equal.")
     #  check for equality of numbers of functions
     if (ndim1 > 2 && ndim2 > 2 && ndim1 != ndim2)
-        stop(paste("Both arguments multivariate, ",
-                   "but involve different numbers ",
-                   "of functions."))
+      stop(paste("Both arguments multivariate, ",
+                 "but involve different numbers ",
+                 "of functions."))
     basisobj1 <- e1$basis
     basisobj2 <- e2$basis
     #  check for equality of two bases
     if (basisobj1 == basisobj2) {
         #  if equal, just difference coefficient matrices
-        fdnames <- e1$fdnames
-        plusfd <- fd(coef1 + coef2, basisobj1, fdnames)
-        return(plusfd)
+      fdnames <- e1$fdnames
+      plusfd <- fd(coef1 + coef2, basisobj1, fdnames)
+      return(plusfd)
     } else {
-        nbasis1   <- basisobj1$nbasis
-        nbasis2   <- basisobj2$nbasis
-        rangeval1 <- basisobj1$rangeval
-        rangeval2 <- basisobj2$rangeval
-        if (any(rangeval1 != rangeval2))
-            stop("The ranges of the arguments are not equal.")
-        neval     <- max(10*max(nbasis1+nbasis2) + 1, 201)
-        evalarg   <- seq(rangeval1[1], rangeval2[2], len=neval)
-        fdarray1  <- eval.fd(e1, evalarg)
-        fdarray2  <- eval.fd(e2, evalarg)
-        if ((ndim1 <= 2 && ndim2 <= 2) ||
-            (ndim1 >  2 && ndim2 >  2))
-            fdarray <- fdarray1 + fdarray2
-        if (ndim1 == 2 && ndim2 > 2) {
-            fdarray <- array(0,coefd2)
-            for (ivar in 1:coefd2[3])
-                fdarray[,,ivar] <- fdarray1 + fdarray2[,,ivar]
-        }
-        if (ndim1 > 2 && ndim2 == 2) {
-            fdarray <- array(0,coefd1)
-            for (ivar  in  1:coefd1[3])
-                fdarray[,,ivar] <- fdarray1[,,ivar] + fdarray2
-        }
-        #  set up basis for sum
-        coefsum  <- project.basis(fdarray, evalarg, basisobj, 1)
-        fdnames1 <- e1$fdnames
-        fdnames2 <- e2$fdnames
-        fdnames  <- fdnames1
-        fdnames[[3]] <- paste(fdnames1[[3]],"+",fdnames2[[3]])
+      nbasis1   <- basisobj1$nbasis
+      nbasis2   <- basisobj2$nbasis
+      rangeval1 <- basisobj1$rangeval
+      rangeval2 <- basisobj2$rangeval
+      if (any(rangeval1 != rangeval2))
+        stop("The ranges of the arguments are not equal.")
+      neval     <- max(10*max(nbasis1+nbasis2) + 1, 201)
+      evalarg   <- seq(rangeval1[1], rangeval2[2], len=neval)
+      fdarray1  <- eval.fd(e1, evalarg)
+      fdarray2  <- eval.fd(e2, evalarg)
+      if ((ndim1 <= 2 && ndim2 <= 2) ||
+          (ndim1 >  2 && ndim2 >  2))
+        fdarray <- fdarray1 + fdarray2
+      if (ndim1 == 2 && ndim2 > 2) {
+        fdarray <- array(0,coefd2)
+        for (ivar in 1:coefd2[3])
+          fdarray[,,ivar] <- fdarray1 + fdarray2[,,ivar]
+      }
+      if (ndim1 > 2 && ndim2 == 2) {
+        fdarray <- array(0,coefd1)
+        for (ivar  in  1:coefd1[3])
+          fdarray[,,ivar] <- fdarray1[,,ivar] + fdarray2
+      }
+#  set up basis for sum
+      coefsum  <- project.basis(fdarray, evalarg, basisobj, 1)
+      fdnames1 <- e1$fdnames
+      fdnames2 <- e2$fdnames
+      fdnames  <- fdnames1
+      fdnames[[3]] <- paste(fdnames1[[3]],"+",fdnames2[[3]])
     }
- } else {
+  } else {
     #  one argument is numeric and the other is functional
     if (!(is.numeric(e1) || is.numeric(e2)))
-        stop("Neither argument for + is numeric.")
+      stop("Neither argument for + is numeric.")
 #    if (is.numeric(e1) && isa_fd(e2)) {
     if (is.numeric(e1) && is.fd(e2)) {
-        fac   <- e1
-        fdobj <- e2
+      fac   <- e1
+      fdobj <- e2
     } else if (is.fd(e1) && is.numeric(e2)) {
-        fac   <- e2
-        fdobj <- e1
+      fac   <- e2
+      fdobj <- e1
     } else
-        stop("One of the arguments for + is of the wrong class.")
+    stop("One of the arguments for + is of the wrong class.")
     coef     <- fdobj$coefs
     coefd    <- dim(coef)
     basisobj <- fdobj$basis
@@ -335,39 +354,39 @@ if (inherits(e1, "fd") && inherits(e2, "fd")) {
     #  is longer, expand the scalar object into a vector
 
     if( length(fac) > 1){
-    	 if (length(fac) > 1 && coefd[2] == 1) {
-           fdmat <- outer(fdmat,rep(1,length(fac)))
-           fac   <- t(outer(rep(neval,1),fac))
-     	  }
-     	  if (length(fac) == coefd[2]){
-	  	fac = t(outer(rep(neval,1),fac))}
-	  if( coefd[2]>1 && length(fac) !=coefd[2] ){
-		stop(paste("Dimensions of numerical factor and functional",
-                       "factor cannot be reconciled."))
-	  }
-     }
+      if (length(fac) > 1 && coefd[2] == 1) {
+        fdmat <- outer(fdmat,rep(1,length(fac)))
+        fac   <- t(outer(rep(neval,1),fac))
+      }
+      if (length(fac) == coefd[2]){
+        fac = t(outer(rep(neval,1),fac))}
+      if( coefd[2]>1 && length(fac) !=coefd[2] ){
+        stop(paste("Dimensions of numerical factor and functional",
+                   "factor cannot be reconciled."))
+      }
+    }
 
     fdarray <- fac + fdmat
     coefsum <- project.basis(fdarray, evalarg, basisobj)
     fdnames <- fdobj$fdnames
     if (length(fac) == 1)
-        fdnames[[3]] <- paste(fac," + ",fdnames[[3]])
-}
+      fdnames[[3]] <- paste(fac," + ",fdnames[[3]])
+  }
 
-plusfd <- fd(coefsum, basisobj, fdnames)
-return(plusfd)
+  plusfd <- fd(coefsum, basisobj, fdnames)
+  return(plusfd)
 
 }
 
-#  ----------------------------------------------------------------------
+#  ---------------------------------------------------------------
 #  minus method for "fd"
-#  --------------------------------------------------------------------------
+#  ---------------------------------------------------------------
 
 "-.fd" <- function(e1, e2){
   minus.fd(e1, e2)
 }
 
-minus.fd <- function(e1, e2, basisobj=basisobj1*basisobj2)
+minus.fd <- function(e1, e2, basisobj=NULL)
 {
 #  MINUS: Pointwise difference two functional data objects,
 #    the between a scalar and a functional data object,
@@ -391,8 +410,8 @@ minus.fd <- function(e1, e2, basisobj=basisobj1*basisobj2)
 #  coefficient matrices are the same dims, the coefficient
 #  matrices are simply added.
 
-#  last modified 2008.09.30 by Giles Hooker
-#  Previously modified 3 January 2007
+# Last modified 2008.12.27 by Spencer Graved
+# Previously modified 2008.09.30 by Giles Hooker
 
   if(missing(e2)){
     if(!inherits(e1, 'fd'))
@@ -431,6 +450,7 @@ if (inherits(e1, "fd") && inherits(e2, "fd")) {
         minusfd  <- fd(coef1 - coef2, basisobj1, fdnames)
         return(minusfd)
     }
+    basisobj <-  basisobj1 * basisobj2
     #  check to see if (the number of dimensions match
     ndim1  <- length(coefd1)
     ndim2  <- length(coefd2)
@@ -564,13 +584,13 @@ return(minusfd)
 
 #  -----------------------------------------------------------------
 #  point-wise product method for "fd"
-#  --------------------------------------------------------------------------
+#  -----------------------------------------------------------------
 
 "*.fd" <- function(e1, e2){
   times.fd(e1, e2)
 }
 
-times.fd <- function(e1, e2, basisobj=basisobj1*basisobj2)
+times.fd <- function(e1, e2, basisobj=NULL)
 {
 #  TIMES: Pointwise product of two functional data objects,
 #    the product of a scalar and a functional data object,
@@ -599,18 +619,19 @@ times.fd <- function(e1, e2, basisobj=basisobj1*basisobj2)
 #  Returns:
 #  FDAPROD  ...  A functional data object that is e1 times e2
 
-#  Last modified:  3 January 2007
+#  Last modified 2008.12.27 by Spencer Graves
+#  previously modified:  3 January 2007
 
 # Check if at least one argument is a functional data object
 
-if ((!(inherits(e1, "fd") | inherits(e2, "fd")))) stop(
-   "Neither argument for * is a functional data object.")
+  if ((!(inherits(e1, "fd") | inherits(e2, "fd"))))
+    stop("Neither argument for * is a functional data object.")
 
 #  Determine which of two cases hold:
 #   1.  both variables are functional
 #   2.  only one of them is functional
 
-if ( inherits(e1, "fd") & inherits(e2, "fd") ) {
+  if ( inherits(e1, "fd") & inherits(e2, "fd") ) {
 
     #  --------------------------------------------------------
     #       both arguments are functional data objects
@@ -629,8 +650,8 @@ if ( inherits(e1, "fd") & inherits(e2, "fd") ) {
     #  check that the two coefficient arrays have the same
     #  number of dimensions
 
-    if (length(coefd1) != length(coefd2)) stop(
-        "Number of dimensions of coefficient arrays do not match.")
+    if (length(coefd1) != length(coefd2))
+      stop("Number of dimensions of coefficient arrays do not match.")
 
     #  allow for one function having a single replicate,
     #  and if so, copy it as many times as there are replicates
@@ -640,48 +661,49 @@ if ( inherits(e1, "fd") & inherits(e2, "fd") ) {
     #  e1 is single,  e2 has replications
 
     if (coefd1[2] == 1 && coefd2[2] > 1) {
-        if     (ndim1 == 2) {
-           coef1 <- matrix(coef1,coefd1[1],coefd2[2])
-         } else if (ndim1 == 3) {
-            temp <- array(0,coefd2)
-            for (j in 1:coefd1[3])
-                temp[,,j] <- outer(coef1[,1,j],rep(1,coefd2[2]))
-            coef1 <- temp
-        } else {
-            stop("Dimensions of coefficient matrices not compatible.")
-        }
-        coefd1       <- dim(coef1)
-        e1$coefs <- coef1
+      if     (ndim1 == 2) {
+        coef1 <- matrix(coef1,coefd1[1],coefd2[2])
+      } else if (ndim1 == 3) {
+        temp <- array(0,coefd2)
+        for (j in 1:coefd1[3])
+          temp[,,j] <- outer(coef1[,1,j],rep(1,coefd2[2]))
+        coef1 <- temp
+      } else {
+        stop("Dimensions of coefficient matrices not compatible.")
+      }
+      coefd1       <- dim(coef1)
+      e1$coefs <- coef1
     }
 
     #  e2 is single,  e1 has replications
 
     if (coefd1[2] >  1 && coefd2[2] == 1) {
 
-        if      (ndim2 == 2) {
-            coef2 <- matrix(coef2,coefd2[1],coefd1[2])
-        } else if (ndim1 == 3) {
-            temp <- array(0,coefd1)
-            for (j in 1:coefd2[3])
-                temp[,,j] <- outer(coef2[,1,j],rep(1,coefd1[2]))
-            coef2 <- temp
-        } else {
-            stop("Dimensions of coefficient matrices not compatible.")
-        }
-        coefd2       <- dim(coef2)
-        e2$coefs <- coef2
+      if      (ndim2 == 2) {
+        coef2 <- matrix(coef2,coefd2[1],coefd1[2])
+      } else if (ndim1 == 3) {
+        temp <- array(0,coefd1)
+        for (j in 1:coefd2[3])
+          temp[,,j] <- outer(coef2[,1,j],rep(1,coefd1[2]))
+        coef2 <- temp
+      } else {
+        stop("Dimensions of coefficient matrices not compatible.")
+      }
+      coefd2       <- dim(coef2)
+      e2$coefs <- coef2
     }
 
     #  check that numbers of replications are equal
 
-    if (coefd1[2] != coefd2[2]) stop("Number of replications are not equal.")
+    if (coefd1[2] != coefd2[2])
+      stop("Number of replications are not equal.")
 
     #  check for matching in the multivariate case
 
     if (ndim1 > 2 && ndim2 > 2 && ndim1 != ndim2)
-        stop(paste("Both arguments multivariate, ",
-                   "but involve different numbers ",
-                   "of functions."))
+      stop(paste("Both arguments multivariate, ",
+                 "but involve different numbers ",
+                 "of functions."))
 
     #  extract the two bases
 
@@ -689,14 +711,19 @@ if ( inherits(e1, "fd") & inherits(e2, "fd") ) {
     basisobj2 <- e2$basis
     nbasis1   <- basisobj1$nbasis
     nbasis2   <- basisobj2$nbasis
-    if (basisobj1 == basisobj2) basisobj = basisobj1
+
+    if(is.null(basisobj))
+       basisobj <- {
+         if (basisobj1 == basisobj2) basisobj1
+         else basisobj1*basisobj2
+       }
 
     #  check that the ranges match if a range not supplied
 
     rangeval1 <- basisobj1$rangeval
     rangeval2 <- basisobj2$rangeval
-    if (any(rangeval1 != rangeval2)) stop(
-        "The ranges of the arguments are not equal.")
+    if (any(rangeval1 != rangeval2))
+      stop("The ranges of the arguments are not equal.")
 
     #  set up a fine mesh for evaluating the product
 
@@ -712,19 +739,19 @@ if ( inherits(e1, "fd") & inherits(e2, "fd") ) {
 
     if ((ndim1 <= 2 && ndim2 <= 2) || (ndim1 > 2 && ndim2 > 2)) {
         #  product array where the number of dimensions match
-        fdarray = fdarray1*fdarray2
+      fdarray = fdarray1*fdarray2
     } else {
         #  product array where the number of dimensions don't match
-        if (ndim1 == 2 && ndim2 > 2) {
-            fdarray = array(0,coefd2)
-            for (ivar in 1:coefd2[3])
-                fdarray[,,ivar] <- fdarray1*fdarray2[,,ivar]
-        }
-        if (ndim1 > 2 && ndim2 == 2) {
-            fdarray = array(0,coefd1)
-            for (ivar in 1:coefd1[3])
-                fdarray[,,ivar] <- fdarray1[,,ivar]*fdarray2
-        }
+      if (ndim1 == 2 && ndim2 > 2) {
+        fdarray = array(0,coefd2)
+        for (ivar in 1:coefd2[3])
+          fdarray[,,ivar] <- fdarray1*fdarray2[,,ivar]
+      }
+      if (ndim1 > 2 && ndim2 == 2) {
+        fdarray = array(0,coefd1)
+        for (ivar in 1:coefd1[3])
+          fdarray[,,ivar] <- fdarray1[,,ivar]*fdarray2
+      }
     }
 
     #  set up the coefficient by projecting on to the
@@ -739,21 +766,21 @@ if ( inherits(e1, "fd") & inherits(e2, "fd") ) {
     fdnames  <- fdnames1
     fdnames[[3]] <- paste(fdnames1[[3]],"*",fdnames2[[3]])
 
-} else {
+  } else {
 
     #  --------------------------------------------------------
     #    one argument is numeric and the other is functional
     #  --------------------------------------------------------
 
-    if ((!(is.numeric(e1) || is.numeric(e2)))) stop(
-        "Neither argument for * is numeric.")
+    if ((!(is.numeric(e1) || is.numeric(e2))))
+      stop("Neither argument for * is numeric.")
     #  make the numerical factor the first objec
     if        (is.numeric(e1) && inherits(e2, "fd")) {
-        fac   <- e1
-        fdobj <- e2
+      fac   <- e1
+      fdobj <- e2
     } else if (is.numeric(e2) && inherits(e1, "fd")) {
-        fac   <- e2
-        fdobj <- e1
+      fac   <- e2
+      fdobj <- e1
     } else stop("One of the arguments for * is of the wrong class.")
     coef     <- fdobj$coefs
     coefd    <- dim(coef)
@@ -770,27 +797,27 @@ if ( inherits(e1, "fd") & inherits(e2, "fd") ) {
     fdnames  <- fdobj$fdnames
     fdnames[[3]] <- paste(as.character(fac),"*",fdnames[[3]])
 
-}
+  }
 
 #  set up the functional data object
 
-fdprod   <- fd(coefprod, basisobj, fdnames)
+  fdprod   <- fd(coefprod, basisobj, fdnames)
 
-return(fdprod)
+  return(fdprod)
 
 }
 
-#  --------------------------------------------------------------------------
+#  -------------------------------------------------------------------
 #  power method for "fd"
-#  --------------------------------------------------------------------------
+#  -------------------------------------------------------------------
 
 "^.fd" <- function(fdobj, power)
 {
-    #  Arguments:
-    #  FDOBJ  ...  A functional data object
-    #  POWER  ...  An exponent
-    #  Returns:
-    #  FDAPOWR  ...  A functional data object that is FD to the power POWER
+#  Arguments:
+#  FDOBJ  ...  A functional data object
+#  POWER  ...  An exponent
+#  Returns:
+#  FDAPOWR  ...  A functional data object that is FD to the power POWER
 
     #  Last modified:  17 September 2005
 
@@ -812,17 +839,17 @@ return(fdprod)
     fdpowr
 }
 
-#  --------------------------------------------------------------------------
+#  -------------------------------------------------------------------
 #  sqrt method for "fd"
-#  --------------------------------------------------------------------------
+#  -------------------------------------------------------------------
 
 sqrt.fd <- function(x)
 {
-    #  Arguments:
-    #  x ...  A functional data object
-    #  Returns:
-    #  FDAROOT  ...  A functional data object that is the square root of x
-    #  Last modified:  17 September 2005
+#  Arguments:
+#  x ...  A functional data object
+#  Returns:
+#  FDAROOT  ...  A functional data object that is the square root of x
+#  Last modified:  17 September 2005
 
     if ((!(inherits(x, "fd")))) stop(
       "First argument for ^ is not a functional data object.")
@@ -840,9 +867,9 @@ sqrt.fd <- function(x)
     return(fdroot)
 }
 
-#  -----------------------------------------------------------------------
+#  ----------------------------------------------------------------
 #       mean for fd class
-#  -----------------------------------------------------------------------
+#  ----------------------------------------------------------------
 
 mean.fd <- function(x, ...)
 {
@@ -871,9 +898,9 @@ mean.fd <- function(x, ...)
   meanfd
 }
 
-#  -----------------------------------------------------------------------
+#  ----------------------------------------------------------------
 #       sum for fd class
-#  -----------------------------------------------------------------------
+#  ----------------------------------------------------------------
 
 sum.fd <- function(..., na.rm=FALSE)
 {
@@ -883,7 +910,8 @@ sum.fd <- function(..., na.rm=FALSE)
 
   fd <- list(...)[[1]]
 
-  if (!(inherits(fd, "fd"))) stop("Argument FD not a functional data object.")
+  if (!(inherits(fd, "fd")))
+    stop("Argument FD not a functional data object.")
 
   coef   <- fd$coefs
   coefd  <- dim(coef)
@@ -908,9 +936,9 @@ sum.fd <- function(..., na.rm=FALSE)
   sumfd
 }
 
-#  -----------------------------------------------------------------------
+#  ----------------------------------------------------------------
 #       c for fd class
-#  -----------------------------------------------------------------------
+#  ----------------------------------------------------------------
 
 "c.fd"<- function(...)
 {
@@ -922,48 +950,47 @@ sum.fd <- function(..., na.rm=FALSE)
 
 #  Last modified 17 September 2005
 
-  	fdlist <- list(...)
-  	n      <- length(fdlist)
-  	fd1    <- fdlist[[1]]
-  	if (n == 1) return(fd1)
-  	coef    <- fd1$coefs
-  	coefd   <- dim(coef)
-  	ndim    <- length(coefd)
-  	basisfd <- fd1$basis
-  	fdnames <- fd1$fdnames
+  fdlist <- list(...)
+  n      <- length(fdlist)
+  fd1    <- fdlist[[1]]
+  if (n == 1) return(fd1)
+  coef    <- fd1$coefs
+  coefd   <- dim(coef)
+  ndim    <- length(coefd)
+  basisfd <- fd1$basis
+  fdnames <- fd1$fdnames
   	#  check that the fd objects are consistent with each other
-  	if(!inherits(fd1, "fd")) stop("Objects must be of class fd")
-  	for(j in (2:n)) {
-    	fdj <- fdlist[[j]]
-    	if(!inherits(fdj, "fd")) stop("Objects must be of class fd")
-    	if(any(unlist(fdj$basis) != unlist(basisfd)))
-      		stop("Objects must all have the same basis")
-    	if(length(dim(fdj$coefs)) != ndim)
-      		stop("Objects must all have the same number of multiple functions")
-  	}
+  if(!inherits(fd1, "fd")) stop("Objects must be of class fd")
+  for(j in (2:n)) {
+    fdj <- fdlist[[j]]
+    if(!inherits(fdj, "fd")) stop("Objects must be of class fd")
+    if(any(unlist(fdj$basis) != unlist(basisfd)))
+      stop("Objects must all have the same basis")
+    if(length(dim(fdj$coefs)) != ndim)
+      stop("Objects must all have the same number of multiple functions")
+  }
   	#  concatenate by concatenate coefficient matrices
-  	if (ndim == 2) {
-    	for (j in 2:n) {
-      		nameslist <- dimnames(coef)
-      		fdj       <- fdlist[[j]]
-      		coefj     <- fdj$coefs
-      		coef      <- cbind(coef, coefj)
-      		nameslist[[2]] <- c(nameslist[[2]], dimnames(coefj)[[2]])
-    	}
-  	} else {
-    	for(j in (2:n)) {
-      		nameslist <- dimnames(coef)
-      		fdj       <- fdlist[[j]]
-      		coefj     <- fdj$coefs
-      		coef      <- c(coef, aperm(coefj, c(1, 3, 2)))
-      		nameslist[[2]] <- c(nameslist[[2]], dimnames(coefj)[[2]])
-    	}
-    	dim(coef) <- c(coefd[1], coefd[3],
-				length(coef)/(coefd[1] * coefd[3]))
-   		coef <- aperm(coef, c(1, 3, 2))
+  if (ndim == 2) {
+    for (j in 2:n) {
+      nameslist <- dimnames(coef)
+      fdj       <- fdlist[[j]]
+      coefj     <- fdj$coefs
+      coef      <- cbind(coef, coefj)
+      nameslist[[2]] <- c(nameslist[[2]], dimnames(coefj)[[2]])
+    }
+  } else {
+    for(j in (2:n)) {
+      nameslist <- dimnames(coef)
+      fdj       <- fdlist[[j]]
+      coefj     <- fdj$coefs
+      coef      <- c(coef, aperm(coefj, c(1, 3, 2)))
+      nameslist[[2]] <- c(nameslist[[2]], dimnames(coefj)[[2]])
+    }
+    dim(coef) <- c(coefd[1], coefd[3],
+                   length(coef)/(coefd[1] * coefd[3]))
+    coef <- aperm(coef, c(1, 3, 2))
   }
   dimnames(coef) <- nameslist
   concatfd <- fd(coef, basisfd, fdnames)
   return(concatfd)
 }
-
