@@ -1,4 +1,4 @@
-"[.fd" <- function(fdobj, i=TRUE, j=TRUE, drop=FALSE) {
+"[.fd" <- function(fdobj, i=TRUE, j=TRUE, drop=TRUE) {
   #  select subsets of curves in a functional data object
 
   coef <- fdobj$coefs
@@ -6,12 +6,12 @@
   ndim <- length(dim(coef))
 
   if(ndim == 2) {
-    coefselect <- coef[, i]
+    coefselect <- coef[, i,drop=FALSE]
     if(length(fdnames[[2]])>1){
 	    fdnames[[2]] = fdnames[[2]][i]
     }
   } else {
-    coefselect <- coef[, i, j]
+    coefselect <- coef[, i, j,drop=drop]
     if(length(fdnames[[2]])>1){
 	    fdnames[[2]] = fdnames[[2]][i]
     }
